#!/usr/bin/env python

"""
View annotated sources.

Copyright (C) 2006, 2007, 2010 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from micropython.data import *
from compiler.visitor import ASTVisitor
import sys
import os
import textwrap

# Classes.

# HTML-related output production.

html_header = """<?xml version="1.0" encoding="iso-8859-15"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
  <title>Module</title>
  <style type="text/css">
    html {
      background-color: black; color: white;
    }

    body {
      padding-bottom: 4em;
      font-size: 14pt; font-family: monospace;
      background-color: black; color: white;
    }

    .nowrap { white-space: nowrap; }

    .class { margin-top: 1em; margin-bottom: 1em; }
    .function { margin-top: 1em; margin-bottom: 1em; }
    .body { padding-left: 2em; }
    .keyword { color: yellow; }
    .comment { color: blue; }
    .class-name { color: cyan; }
    .function-name { color: cyan; }
    .str { color: #FF00FF; }
    .doc { color: #FF00FF; margin-top: 1em; margin-bottom: 1em; }
    .invocation a { color: white; text-decoration: none; }

    .popup {
      display: none; z-index: 2;
      position: absolute; top: 2ex; left: 0;
      padding: 0.2em; background-color: #000000; color: white;
      border: 2px solid #dddddd;
    }

    .invocations {
      padding: 0.5em; background-color: #770000;
      clear: all;
    }

    .types {
      padding: 0.5em; background-color: #0000FF;
      float: right;
    }

    .raises {
      padding: 0.5em; background-color: #7700FF;
      float: right;
    }

    .scopes {
      padding: 0.5em; background-color: #007700;
      float: left;
    }

    .non-scopes {
      padding: 0.5em; background-color: #FF0000;
      float: left;
    }

    .no-types {
      background-color: #FF0000;
    }

    .op,
    .name,
    .attr,
    .conditional,
    .operator,
    .iterator,
    .call,
    .returns,
    .failure
    {
      position: relative;
    }

    .op:hover > .popup,
    .name:hover > .popup,
    .attr:hover > .popup,
    .conditional:hover > .popup,
    .operator:hover > .popup,
    .iterator:hover > .popup,
    .call:hover > .popup,
    .returns:hover > .popup,
    .failure:hover > .popup
    {
      display: block;
    }

    .summary-class {
      background-color: #004400;
    }

    .summary-instance {
      background-color: #0000FF;
    }

    .summary-attr {
      background-color: #007700;
    }

  </style>
</head>
<body>
"""

html_footer = """</body>
</html>
"""

# Utility classes.

class Writer:

    "A utility class providing useful HTML output methods."

    # Methods which return strings.

    def _text(self, text):
        return text.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")

    def _attr(self, attr):
        return self._text(attr).replace("'", "&apos;").replace('"', "&quot;")

    def _url(self, url):
        return self._attr(url).replace("#", "%23").replace("-", "%2d")

    def _summary_link(self, module_name, name):
        return "<a href='%s-summary%sxhtml#%s'>%s</a>" % (module_name, os.path.extsep, self._attr(name), self._text(name))

    # Methods which write to the stream.

    def _comment(self, comment):
        self.stream.write("<span class='comment'># %s</span>\n" % comment)

    def _keyword(self, kw):
        self.stream.write("<span class='keyword'>%s</span> " % kw)

    def _doc(self, node):
        if node.doc is not None:
            self.stream.write("<pre class='doc'>\n")
            self.stream.write('"""')
            output = textwrap.dedent(node.doc.replace('"""', '\\"\\"\\"'))
            self.stream.write(self._text(output))
            self.stream.write('"""')
            self.stream.write("</pre>\n")

    def _name(self, name):
        self.stream.write("<span class='name'>%s</span>\n" % name)

# Summary classes.

class Summariser(Writer):

    "Summarise classes and attributes in modules."

    def __init__(self, module):
        self.module = module

    def to_stream(self, stream):

        "Write the summary to the given 'stream'."

        self.stream = stream
        self._init_details()
        self.stream.write(html_header)
        self._write_classes(self.module)
        self.stream.write(html_footer)

    def _write_classes(self, module):
        self.stream.write("<table cellspacing='5' cellpadding='5'>\n")

        all_classes = {}

        for obj in self.module.all_objects:
            if isinstance(obj, Class):
                all_classes[obj.name] = obj

        all_class_names = all_classes.keys()
        all_class_names.sort()

        for name in all_class_names:
            self._write_class(all_classes[name])

        self.stream.write("</table>\n")

    def _write_class(self, obj):

        # Write the class...

        self.stream.write("<tbody class='class'>\n")
        self.stream.write("<tr>\n")
        self.stream.write("<th class='summary-class' id='%s'>\n" % obj.full_name())
        self._keyword("class")
        self.stream.write(obj.name)
        self.stream.write("</th>\n")

        # ...and all known attribute names.

        obj_attributes = obj.all_attribute_names()

        for name in self.attribute_names:
            if name in obj_attributes:
                self.stream.write("<th class='summary-attr'>%s</th>\n" % self._text(name))
            else:
                self.stream.write("<th></th>\n")
        self.stream.write("</tr>\n")

        self.stream.write("</tbody>\n")

    def _init_details(self):
        names = set()

        # Visit all classes.

        for obj in self.module.all_objects:
            if isinstance(obj, Class):
                names.update(obj.all_attribute_names())

        self.attribute_names = list(names)
        self.attribute_names.sort()

# Convenience functions.

def summary(module, filename):
    stream = open(filename, "wb")
    try:
        summariser = Summariser(module)
        summariser.to_stream(stream)
    finally:
        stream.close()

# vim: tabstop=4 expandtab shiftwidth=4
